#highscores.py
#07mar02 PG based on miniconsole.py
#See http://www.pygame.org/

#ZZZ pickle/unpickle scores to/from file
#ZZZ send/receive scores from website

#ZZZ attract modes which advertise company and/or mrexcessive
#ZZZ fixed pitch font


try:
   import pickle
   import pygame
   import pygame.draw,pygame.font
   from pygame.locals import *
   import random
except ImportError, err:
   print "couldn't load module. %s" % (err)
   sys.exit(2)

class HighScores:
   def __init__(self, screen, visible=0, font=None, numlines=20, fontsize=18,topleft=(0,0),\
                  fname_highscores='hiscores.txt'):
      self.screen = screen
      #ZZZ shrink mode ?
      if not self.screen: raise RuntimeError, "screen not initialized"
      if font == None:
         if not pygame.font: raise RuntimeError, "no pygame.font module"
         pygame.font.init()
         font = pygame.font.Font('kitchen.ttf', fontsize)
      self.font = font
      self.numlines = numlines
      self.lineheight = font.get_linesize()
      size = self.screen.get_width(), self.lineheight*(numlines+2)
      self.rect = Rect((0, 0), size)
      self.rect.topleft = topleft
      self.visible = visible
      self.clear()
      self.customize()
      self.consolechange = 0
      self.capslock = 0
      self.edit_which_row = -1
      self.fname = fname_highscores
      self.read_scores()
      self.sort()
      self.puttext("HIGH SCORES",0,textcolor=(255,0,0),col=8)
      self.putscores()
      self.entered_maxlen = 6
      
   def sort(self):
      "sort the scores and leave only 10"
      def myCmp(a,b):
         return cmp(b[0],a[0])      # high first
      self.scores.sort(myCmp)
      self.scores[10:] = []      # truncate
   
   def insert(self,score,name='',doedit=1):
      "insert the new score, if useable. Setup to accept name for this score"
      #ZZZ
      self.scores.append((score,name))
      self.sort()    # will drop scores which are too low
      if doedit:
         row = -1
         for i in range(len(self.scores)-1,-1,-1):
            if self.scores[i][0] == score:      # jump to bottom of equal scores (achieved it later)
               row = i
               break
         self.edit_which_row = row     # row or -1
         self.putscores()  # redraw after insertion
      self.write_scores()

   def customize(self,textcolor=(255,200,200),bgcolor=(40,20,20),\
                        entercolor=(255,155,100),event=USEREVENT,\
                        prompt=' '):
      self.textcolor = textcolor
      self.bgcolor = bgcolor
      self.entercolor = entercolor
      self.event = event
      self.prompt = prompt
      
   def set_textcolor(self,textcolor):
      self.textcolor = textcolor

   def clear(self):
      self.buffer = [None for x in range(self.numlines)]
      self.entered = ''
      
   def putentered(self,final=0):
      if not final:
         self.puttext(self.entered + '-',self.edit_which_row+2,col=10,clear=2,textcolor=(255,255,255))     #edit score in white
      elif self.edit_which_row != -1:
         (score,name) = self.scores[self.edit_which_row]     # drop any old name content
         self.scores[self.edit_which_row] = (score,self.entered)
         self.puttext(self.entered,self.edit_which_row+2,col=10,clear=2,textcolor=(255,255,0))  # yellow for newest
         self.edit_which_row = -1
         self.entered = ''
         self.write_scores()

   def putscores(self):
      "display the scores and names. Relies on them already being sorted, if required"
      colours = [(random.randint(150,255),random.randint(150,255),random.randint(150,255)) for i in range(10)]
      use_colours = colours
      row = 0
      for scoreline in self.scores:
         (score,name) = scoreline
         colour = use_colours.pop()
         if not use_colours:
            use_colours = colours
         self.puttext(str(score),row+2,col=8,textcolor=colour,clear=1,flag_justifyright=1,x_offset=30)
         if self.edit_which_row != row:      # display
            self.puttext(name,row+2,col=10,textcolor=colour)
         else:       # edit
            self.putentered()
         row += 1
     
   def puttext(self,text,row,col=0,textcolor=None,x_offset=0,flag_justifyright=0,clear=0):
      if not textcolor:
         textcolor = self.textcolor
      text = text.strip()
      if text:
         try:
            #ZZZ justify right and use x_offset to fine position
            new_image = self.buffer[row]
            colx = col * 45
            if clear == 1 or not new_image:     # clear==1 => clear whole line
               new_image = pygame.Surface((self.rect.width,self.lineheight))
            elif clear == 2:                          # clear==2 => preserve before colx
               new_image.fill((0,0,0),Rect(colx,0,self.rect.width-colx,self.lineheight))
            newtext_image = self.font.render(text,1,textcolor)
            if flag_justifyright:
               colx = colx + x_offset - newtext_image.get_rect().width
            
            new_image.blit(newtext_image,(colx,0))
            self.buffer[row] = new_image
         except Exception,err:
            print "ERR puttext: %s " % (err)
            pass
            
   def write_scores(self):
      pickle.dump(self.scores,open(self.fname,'w'))
   
   def read_scores(self):
      try:
         self.scores = pickle.load(open(self.fname))     # [(score,name)]
      except:
          self.scores = [(3000,'sam'),(4000,'honey'),(2000,'chess'),\
             (850,'digsy'),(2400,'figment'),(1800,'fudge'),\
             (1400,'dooley'),(6000,'bumbles'),(18000,'mcavity')]
          print "Cannot load highscores file"

   def draw(self):
      #ZZZ this doesn't really need to do much... certainly not redraw each time
      if not self.visible: return None
      #self.screen.fill(self.bgcolor, self.rect)      # if you want a background to it all, use this
      height = 0
      for line in self.buffer:
         if line is not None:
            self.screen.blit(line, (self.rect.topleft[0], height + self.rect.topleft[1]))
         height += self.lineheight
      #entry = self.prompt + self.entered XXX remove
      #img = self.font.render(entry, 1, self.entercolor)
      self.screen.set_colorkey((0,0,0))
      #self.screen.blit(img, (3, height+3) + self.rect.topleft)
      return self.rect
      
   def handle(self, event):
      if not self.visible:    # don't catch events
         return event
      if event.type == KEYDOWN:
         if event.key == K_BACKSPACE:
            if self.entered: self.entered = self.entered[:-1]
            self.putentered()
         elif event.key == K_ESCAPE:
            # just skip hiscore name entry
            self.edit_which_row = -1
            #ZZZ redraw? with the name gone
            pass
         elif event.key == K_RETURN:
            if self.entered:
               entered = self.entered
               self.putentered(final=1)      # final==1 => clear self.entered &c.
               return pygame.event.Event(self.event, command=entered)
         elif self.edit_which_row != -1 and event.unicode:
            # we never know where this may blow up, so carefully does it
            newch = ''
            try:
               newch = str(event.unicode)
            except:
               pass     # space should be safe
            if newch and len(self.entered) < self.entered_maxlen:
               self.entered += newch
               self.putentered()
            return pygame.event.Event(NOEVENT)

      return event

   def togglevisible(self):
      self.visible = not self.visible
      self.consolechange = 1     # bus ready


